@@ -7,13 +7,17 @@ import pandas as pd
 from natsort import natsorted
 import os
 from time import sleep
+from tqdm import tqdm
+import warnings
+warnings.filterwarnings('ignore')
+
 
 
 verbose=False
 #verbose=True
 progressmenu=1
 startRow=2
-
+plotOne=['-', 0] # SubPlot to Take as 1 
 plotsDF = pd.DataFrame(columns=['plotNo','subplot','sno','status']) # DataFrame
 n=0 # DataFrame Index
 
@@ -42,8 +46,11 @@ def splitAlpnaNum(sps):
     #import re; return [ i for i in re.split('(\d+)', sps) if i ]
 
 def clear():
+    from IPython import get_ipython
+    get_ipython().run_line_magic('clear','')
     os.system('cls' if os.name=='nt' else 'clear')
-   
+
+
 
 pval='' ; pltseq=[]
 def findMissingPlots(subp): # Takes Subplots of a Plot
@@ -118,14 +125,15 @@ def getsubplots( subplot, pno=0, addPlot=0): # Gives All Possible subplots of a
     '''
     Pass the List having Subplots in random order to get the Sequenced Subplots
     '''
-    global n, plotsDF, MissedSinglePlots
+    global n, plotsDF, plotOne, MissedSinglePlots
     if not addPlot:
         if not pno:
             plotsDF = pd.DataFrame(columns=['plotNo','subplot','sno','status']); n=0; MissedSinglePlots={}
             
             subplot=[int(i) if str(i).isdigit() else str(i) for i in subplot.split(',')] if isinstance(subplot, str) else subplot
+            subplot=[ 1 if i in plotOne else i for i in subplot ]
+
 
-    
         mainRange = max(set([ i if isinstance(i,int) else getNumber(i)  for i in subplot ])) # Get Max. Main Subplots
         SinglePlots = [ str(i) for i in subplot if isinstance(i,int) ] # Single Plots into string
         GivenSubPlots=[ i for i in subplot if isinstance(i,str) ] # All SubPlots are string type
@@ -200,11 +208,11 @@ def getFilename(FILE_NAME, noext=0):
         return FILE_NAME if noext else FILE_NAME+'.xlsx'
 
   
-def createEXCEL(filename, checkfile=0 , autoOpen=1):
+def createEXCEL(filename, checkfile=0 , autoOpen=1, selectedPlots=0):
     global plotsDF, startRow
     wb=openpyxl.Workbook()
     sheet=wb.active
-    sheet.title = 'Final'
+    sheet.title =  'Final'
       
     START_COL='A'
     START_ROW='2'
@@ -251,12 +259,7 @@ def createEXCEL(filename, checkfile=0 , autoOpen=1):
         sheet[chr(ord(START_COL) )+str(int(START_ROW)+i)] = plotsDF.iloc[i,0] # plot No.
         sheet[chr(ord(START_COL)+1)+str(int(START_ROW)+i)]= int(plotsDF.iloc[i,1]) if plotsDF.iloc[i,1].isdigit() else plotsDF.iloc[i,1] # SubPlot
         sheet[chr(ord(START_COL)+2)+str(int(START_ROW)+i)]= plotsDF.iloc[i,2] # S.No.
-        
-        ### Formatting for Newly Added Subplots
-        if plotsDF.iloc[i,3] == 'ADDED' :
-            for j in range(3):
-                sheet[chr(ord(START_COL)+j)+str(int(START_ROW)+i)].border = thin_border
-                sheet[chr(ord(START_COL)+j)+str(int(START_ROW)+i)].fill = PatternFill(fgColor=AddedPlotColour, fill_type = 'solid')
+
 
         ## Add Line on Plot No. Change
         if nextPlotNo != plotsDF.iloc[i,0]:
@@ -264,46 +267,135 @@ def createEXCEL(filename, checkfile=0 , autoOpen=1):
                 sheet[chr(ord(START_COL)+j)+str(int(START_ROW)+i)].border = top_border
         nextPlotNo=plotsDF.iloc[i,0]
         
+        ### Formatting for Newly Added Subplots
+        if plotsDF.iloc[i,3] == 'ADDED' :
+            for j in range(3):
+                sheet[chr(ord(START_COL)+j)+str(int(START_ROW)+i)].fill = PatternFill(fgColor=AddedPlotColour, fill_type = 'solid')
+                sheet[chr(ord(START_COL)+j)+str(int(START_ROW)+i)].border = thin_border
+        
         
         ## Cell Formatting
         sheet[chr(ord(START_COL))+str(int(START_ROW)+i)].alignment = alignment
         sheet[chr(ord(START_COL)+1)+str(int(START_ROW)+i)].alignment = leftalignment
         sheet[chr(ord(START_COL)+2)+str(int(START_ROW)+i)].alignment = alignment
-  
-    
-    sortedPlots = sorted(plotInfo, key=plotInfo.get, reverse=True) # Based on Subplots count
-    print('\n\n     Plot No. | Divisions | Probability/%')
-    for i,plt in enumerate(sortedPlots[:10]):
-        print(' '+'{:>2}'.format(i+1)+'. '+'{:>5}'.format(plt)+'  '+'{:>9}'.format(plotInfo[plt])+'        '+format((plotInfo[plt]/len(plotsDF.index))*100, '.2f')+' %' )
-        #print(' '+'{:>2}'.format(i+1)+'. '+'{:>3}'.format(plt)+'  '+'{:>3}'.format(plotInfo[plt])+'  '+format((plotInfo[plt]/len(plotsDF.index))*100, '.2f')+' %' )
-    if checkfile:
-        wbck = openpyxl.load_workbook(getFilename(checkfile))
-        sheetck=wbck[wbck.sheetnames[-1]]
-        sheetck=wbck.active
-        
-        ROW_MAX=sheetck.max_row
-        
-        chkplots={} # Dictionary to store Plot no. as key & its subplots as value in list form
-        for i in range(startRow,ROW_MAX+1):
-            #print(pno,' : ',spno)
-            pno=sheetck['A'+str(i)].value # plot no.
-            if pno:
-                spno = sheetck['B'+str(i)].value
-                sno = sheetck['C'+str(i)].value
-                chkplots[sno] = spno
-        cn=int(START_ROW)
-        for sno in chkplots:
-            sheet['D'+str(cn)] = sno # SNo.
-            sheet['E'+str(cn)] = chkplots[sno] # SNo.
-            sheet['D'+str(cn)].alignment = alignment; sheet['E'+str(cn)].alignment = alignment
-            cn+=1
+
         
+    if selectedPlots:
+        sheet.title = 'Selected'
+        splots=list(selectedPlots.keys())
+        selectedPlots=[i for l in [selectedPlots[i] for i in selectedPlots] for i in l ]
+        for i in range(len(plotsDF.index)):
+            if plotsDF.iloc[i,2] in selectedPlots:
+                pltcolor = '03A9F4' if sheet[chr(ord(START_COL)+2)+str(int(START_ROW)+i)].value in splots else 'B3E5FC'
+                sheet[chr(ord(START_COL))+str(int(START_ROW)+i)].fill = PatternFill(fgColor=pltcolor, fill_type = 'solid') # 81D4FA
+                
+                sheet[chr(ord(START_COL)+1)+str(int(START_ROW)+i)].fill = PatternFill(fgColor=pltcolor, fill_type = 'solid')
+                sheet[chr(ord(START_COL))+str(int(START_ROW)+i)].border = thin_border
+                sheet[chr(ord(START_COL)+1)+str(int(START_ROW)+i)].border = thin_border
+
+    else:
+        sortedPlots = sorted(plotInfo, key=plotInfo.get, reverse=True) # Based on Subplots count
+        print('\n\n     Plot No. | Divisions | Probability/%')
+        for i,plt in enumerate(sortedPlots[:10]):
+            print(' '+'{:>2}'.format(i+1)+'. '+'{:>5}'.format(plt)+'  '+'{:>9}'.format(plotInfo[plt])+'        '+format((plotInfo[plt]/len(plotsDF.index))*100, '.2f')+' %' )
+            #print(' '+'{:>2}'.format(i+1)+'. '+'{:>3}'.format(plt)+'  '+'{:>3}'.format(plotInfo[plt])+'  '+format((plotInfo[plt]/len(plotsDF.index))*100, '.2f')+' %' )
+        if checkfile:
+            wbck = openpyxl.load_workbook(getFilename(checkfile))
+            sheetck=wbck[wbck.sheetnames[-1]]
+            sheetck=wbck.active
+            
+            ROW_MAX=sheetck.max_row
+            
+            chkplots={} # Dictionary to store Plot no. as key & its subplots as value in list form
+            for i in range(startRow,ROW_MAX+1):
+                #print(pno,' : ',spno)
+                pno=sheetck['A'+str(i)].value # plot no.
+                if pno:
+                    spno = sheetck['B'+str(i)].value
+                    sno = sheetck['C'+str(i)].value
+                    chkplots[sno] = spno
+            cn=int(START_ROW)
+            for sno in chkplots:
+                sheet['D'+str(cn)] = sno # SNo.
+                sheet['E'+str(cn)] = chkplots[sno] # SNo.
+                sheet['D'+str(cn)].alignment = alignment; sheet['E'+str(cn)].alignment = alignment
+                cn+=1
+
     wb.save(filename)  
             
     if autoOpen :
         sleep(4)
         os.system('start EXCEL.EXE "'+filename+'"')
+
+
+def selectplots(plotsDF):
+    stmgrp=''
+    if isinstance(plotsDF,str):
+        tplot,stmgrp=tuple(plotsDF.split(',')) if ',' in plotsDF else (plotsDF,'')
+    elif isinstance(plotsDF,list):
+        tplot,stmgrp=(plotsDF[0],str(plotsDF[1])) if len(plotsDF) == 2 else (plotsDF[0],'')
+    else:   
+        tplot=len(plotsDF.index)
+    tplot=int(tplot)
+    def SelectedPlots(i):
+        startRange=(i-i%10  if 0<i%10<6 else  i-i%5) if i%10 else i-5
+        return [ startRange+i for i in range(1,6) ]    
     
+    rtplot=tplot+abs(tplot%10-10)
+    splotinterval=int(rtplot/4)
+    stmflag=True; grpflag=True
+    print('\n\n')
+    if not stmgrp:
+        while(stmflag):
+            stm=input('\n  - Enter Stratum : ')
+            stmflag=False if stm.isdigit() else True
+        while(grpflag):
+            grp=input('  - Enter Group   : ')
+            grpflag=False if grp.isdigit() else True
+        stmgrp=stm+grp
+
+    RandomTable = pd.read_pickle('RandomTable.pkl')
+    for num in RandomTable[int(stmgrp)]:
+        if int(num) <= tplot:
+            RNUM=int(num)
+            break
+    splots = [ RNUM+splotinterval*i  if RNUM+splotinterval*i <= tplot else RNUM+splotinterval*i-tplot  for i in range(1,5) ] # Four Randomly Selected Plot SNo.
+
+    print('\n\n   Random Number : ',RNUM,'\n   Plot Interval : ',splotinterval)
+    selectedplots={}
+    if isinstance(plotsDF,pd.core.frame.DataFrame):
+        sleep(2)
+        print('\n\n    SELECTED PLOTS :: ')
+        print('\n         SNo.     Plot - Subplot\n')
+        n=1
+        for plt in splots:
+            selectedplots[plt]=SelectedPlots(plt)
+            for sno in selectedplots[plt]:
+                pno=plotsDF[plotsDF['sno']==sno]['plotNo'].values[0]
+                subplt=plotsDF[plotsDF['sno']==sno]['subplot'].values[0]
+                if sno==plt:
+                    print(' [','{:>2}'.format(n)+'.','{:>5}'.format(sno),' ->  ',pno,'-',subplt,' ]')
+                else:
+                    print('  ','{:>2}'.format(n)+'.','{:>5}'.format(sno),' ->  ',pno,'-',subplt)
+                print() if not n%5 else ''
+                n+=1
+    
+        return selectedplots
+    else:
+        spltstr=[]; n=0
+        for plt in splots:
+            selectedplots[plt]=SelectedPlots(plt)
+            for sno in selectedplots[plt]:
+                n+=1
+                if sno==plt:
+                    spltstr.append('  {:>2}'.format(n)+'. [{:>5}'.format(sno)+']')
+                else:
+                    spltstr.append('  {:>2}'.format(n)+'.  {:>5}'.format(sno))
+                spltstr.append('\n') if not n%5 else ''
+        return { 'param':[ RNUM,splotinterval ] , 'splots': spltstr }
+
+        
+
 
 def agriplot(FILE_NAME, checkfile=''):
     '''
@@ -317,7 +409,8 @@ def agriplot(FILE_NAME, checkfile=''):
     ROW_MAX=sheet.max_row
     
     perday=1000
-    plotOne=['-', 0] # SubPlot to Take as 1 
+    progressTime=18
+    global plotOne
     #ROW_MAX=232
     plots={} # Dictionary to store Plot no. as key & its subplots as value in list form
     for i in range(startRow,ROW_MAX+1):
@@ -328,26 +421,39 @@ def agriplot(FILE_NAME, checkfile=''):
             spno = sheet['B'+str(i)].value if sheet['B'+str(i)].value not in plotOne else 1 # its subplot
             plots.setdefault(pno,[])
             plots[pno].append(spno)
-    
+
 
     addedPlots=[]
     pn=0; ln=0
-    for plotNo in range(1,list(plots.keys())[-1]+1):
-        if plots.get(plotNo):
-            pn+=1
-            ln+=len(plots[plotNo])
+    Tplot=sum([ len(plots[pno]) if plots.get(pno) else 1  for pno in range(1,list(plots.keys())[-1]+1) ]) ; ln=0
+    LastPlot=list(plots.keys())[-1] # For  Updatebar increment
+    progressIncT=progressTime/Tplot # Updatebar wait
+    with tqdm(total=100,bar_format='{desc}{percentage:3.1f}% | {bar}  [{elapsed}<{remaining}]' ) as pbar:    
+        for plotNo in range(1,list(plots.keys())[-1]+1):
+            if plots.get(plotNo):
+                pn+=1
+                ln+=len(plots[plotNo]) # Show Original Lines for Days calculation
+                '''
+                    clear()
+                    progress = ('.  ' if pn%2 else ' . ' if pn%3 else '  .' )
+                    
+                    print('\n\n\n      Plot:','{:>3}'.format(plotNo),progress, '{:>5}'.format(int((pn/len(plots))*100)) ,'%  ')
+                    print('\n      Days: ', ( 1 if ln <= perday else int(ln/perday)+1), '  |  Lines:','{:>5}'.format(ln) )
+                '''
+
+                subplots=getsubplots(plots[plotNo], plotNo)
+                #uptL=(len(subplots)/Tplot)*100
+                
+            else:
+                subplots=getsubplots('1' , plotNo, addPlot=plotNo)
+                #ln+=1 # Uncomment to show Added Lines
+                addedPlots.append(plotNo)
             if progressmenu:
-                progress = ('.  ' if pn%2 else ' . ' if pn%3 else '  .' )
-                print('\n\n\n      Plot:','{:>3}'.format(plotNo),progress, '{:>5}'.format(int((pn/len(plots))*100)) ,'%  ')
-                print('\n      Days: ', ( 1 if ln <= 1000 else int(ln/1000)+1), '  |  Lines:','{:>5}'.format(ln) )
-                clear()
+                pbar.set_description("  Days:"+str( 1 if ln <= perday else int(ln/perday)+1)+' | Lines:'+str(ln)+' | Plots:'+str(plotNo)+' ')
+                pbar.update(100/LastPlot)
+                sleep(progressIncT)
+                #sleep(0.005/uptL)
 
-    
-            getsubplots(plots[plotNo], plotNo)
-        else:
-            getsubplots('1' , plotNo, addPlot=plotNo)
-            addedPlots.append(plotNo)
-            ln+=1
 
     print('\n\n\n')
     if MissedSinglePlots:
@@ -355,18 +461,27 @@ def agriplot(FILE_NAME, checkfile=''):
             print('\n  Missing In Plot:',mplt, ' - ',MissedSinglePlots[mplt][0], '  Added: ',[i[:-2] for i in MissedSinglePlots[mplt][1]])
     if addedPlots:
         print('\n\n  Added Single Plots:',addedPlots)
-    
-    
-    #plotsDF[plotsDF['status']=='ADDED']
+        
+    TotalAdded=plotsDF[plotsDF['status']=='ADDED']['status']
+    if not TotalAdded.empty:
+        TotalAddedPlots=int(TotalAdded.value_counts())
+        print('\n\n  TOTAL Plots/SubPlots[Lines] Added: ',TotalAddedPlots,'\n')
+        #print('  TOTAL Plots/SubPlots : ',len(plotsDF.index),' = ',ln,'+',TotalAddedPlots,'\n')
 
-    createEXCEL(str(getFilename(FILE_NAME, noext=1)).upper()+' - Final .xlsx', checkfile=checkfile )
+
+    createEXCEL(str(getFilename(FILE_NAME, noext=1)).upper()+' - Plots .xlsx', checkfile=checkfile, autoOpen=0 )
+    
+    selectedPlots=selectplots(plotsDF)
+    createEXCEL(str(getFilename(FILE_NAME, noext=1)).upper()+' - Selection .xlsx', checkfile=0 , selectedPlots=selectedPlots)
+    
+    return plotsDF
 
 
 if __name__ == '__main__' :
 
-    FILE_NAME='plots.xlsx'
+    #FILE_NAME='tplots.xlsx'
     FILE_NAME='data'
     #FILE_NAME='plot1.xlsx'
-    agriplot(FILE_NAME, checkfile='done' )
-    #agriplot(FILE_NAME )
+    #agriplot(FILE_NAME, checkfile='done' )
+    agriplot(FILE_NAME )
     
